@@ -41,6 +41,7 @@ our $ilen = 4;
 ##    (
 ##     connect=>\%connectArgs,  ##-- passed to IO::Socket::INET->new()
 ##     mode   =>$queryMode,     ##-- one of 'table', 'html', 'text', 'json', or 'raw'; default='json' ('html' is not yet supported)
+##     linger =>\@linger,       ##-- SO_LINGER socket option; default=[1,0]: immediate termination
 ##     ##
 ##     ##-- hit parsing options
 ##     parseMeta=>$bool,        ##-- if true, hit metadata will be parsed to %$hit (default=1)
@@ -82,6 +83,7 @@ sub new {
   my $dc =bless {
 		 ##-- connection args
 		 connect=>\%connect,
+		 linger => [1,0],
 		 mode   =>'json',
 		 encoding => 'UTF-8',
 
@@ -222,6 +224,7 @@ sub open {
   my $dc = shift;
   $dc->{sock} = IO::Socket::INET->new(%{$dc->{'connect'}})
     or return undef;
+  $dc->{sock}->setsockopt(SOL_SOCKET, SO_LINGER, pack('II',@{$dc->{linger}})) if ($dc->{linger});
   $dc->{sock}->autoflush(1);
   return $dc->{sock};
 }
@@ -234,7 +237,93 @@ sub close {
 }
 
 ##======================================================================
-## Query: print(), read*()
+## Common Requests
+
+## $rsp = $dc->request($request_string)
+sub request {
+  my $dc = shift;
+  $dc->send($_[0]);
+  my $buf = $dc->readData();
+  $dc->close();
+  return $buf;
+}
+
+## $data = $dc->requestJson($request_string)
+sub requestJson {
+  my $dc  = shift;
+  return $dc->decodeJson($dc->request($_[0]));
+}
+
+## $server_version = $dc->version()
+sub version {
+  my $dc = shift;
+  return $dc->request("version");
+}
+
+## $status = $dc->status()
+## $status = $dc->status($timeout)
+sub status {
+  my ($dc,$timeout) = @_;
+  $timeout = $dc->{timeout} if (!defined($timeout));
+  return $dc->requestJson("status".(defined($timeout) ? " $timeout" : ''));
+}
+
+## $vstatus = $dc->vstatus()
+## $vstatus = $dc->vstatus($timeout)
+sub vstatus {
+  my ($dc,$timeout) = @_;
+  $timeout = $dc->{timeout} if (!defined($timeout));
+  return $dc->requestJson("vstatus".(defined($timeout) ? " $timeout" : ''));
+}
+
+## $info = $dc->info()
+## $info = $dc->info($timeout)
+sub info {
+  my ($dc,$timeout) = @_;
+  $timeout = $dc->{timeout} if (!defined($timeout));
+  return $dc->requestJson("info".(defined($timeout) ? " $timeout" : ''));
+}
+
+## $expandRaw = $dc->expand_terms($pipeline, $term)
+## $expandRaw = $dc->expand_terms($pipeline, $term, $timeout)
+## $expandRaw = $dc->expand_terms($pipeline, $term, $timeout, $subcorpus)
+## $expandRaw = $dc->expand_terms(\@pipeline, \@terms)
+## $expandRaw = $dc->expand_terms(\@pipeline, \@terms, $timeout)
+## $expandRaw = $dc->expand_terms(\@pipeline, \@terms, $timeout, $subcorpus)
+sub expand_terms {
+  my ($dc,$chain,$terms,$timeout,$subcorpus) = @_;
+  $chain = join('|', @$chain)  if (UNIVERSAL::isa($chain,'ARRAY'));
+  $terms = join("\t", @$terms) if (UNIVERSAL::isa($terms,'ARRAY'));
+
+  ##-- hack: detect swapping of $timeout and $subcorpus (old DDC::Client::Distributed-style)
+  $timeout   = '' if (!defined($timeout));
+  $subcorpus = '' if (!defined($subcorpus));
+  ($timeout,$subcorpus) = ($subcorpus,$timeout)
+    if ($timeout ne '' && $subcorpus ne '' && $timeout =~ /[0-9]/ && $subcorpus !~ /[0-9]/);
+
+  $timeout   = $dc->{timeout} if ($timeout eq '');
+  $timeout   = 5 if (!defined($timeout) || $timeout eq '');
+  $dc->send(join("\x01", 'expand_terms ', $chain, $terms, $timeout, $subcorpus));
+  ##-- get output buffer
+  my $buf = $dc->readData();
+  $dc->close(); ##-- this apparently has to happen: bummer
+  return $buf;
+}
+
+## \@terms = $dc->expand($pipeline, $term)
+## \@terms = $dc->expand($pipeline, $term, $timeout)
+## \@terms = $dc->expand($pipeline, $term, $timeout, $subcorpus)
+## \@terms = $dc->expand(\@pipeline, \@terms)
+## \@terms = $dc->expand(\@pipeline, \@terms, $timeout)
+## \@terms = $dc->expand(\@pipeline, \@terms, $timeout, $subcorpus)
+sub expand {
+  my $dc = shift;
+  return $dc->parseExpandTermsResponse($dc->expand_terms(@_));
+}
+
+
+##======================================================================
+## Low-level communications
 
 ## $encoded = $dc->ddc_encode(@message_strings)
 sub ddc_encode {
@@ -557,6 +646,18 @@ sub parseJsonData {
   return $hits;
 }
 
+##--------------------------------------------------------------
+## Hit Parsing: expand_terms()
+
+## \@terms = $dc->parseExpandTermsResponse($buf)
+##  @terms = $dc->parseExpandTermsResponse($buf)
+sub parseExpandTermsResponse {
+  my $dc    = shift;
+  my @items = grep {defined($_) && $_ ne ''} split(/[\t\r\n]+/,$_[0]);
+  die("error in expand_terms response") if ($items[0] !~ /^0 /);
+  shift(@items);
+  return wantarray ? @items : \@items;
+}
 
 1; ##-- be happy
 
@@ -580,7 +681,27 @@ DDC::Client - Client socket utilities for DDC::Concordance
  $dc = DDC::Client->new(PeerAddr=>'localhost',PeerPort=>50000);
 
  ##---------------------------------------------------------------------
- ## Low-level communications
+ ## open, close
+
+ $io_socket = $dc->open();
+ undef      = $dc->close();
+
+ ##---------------------------------------------------------------------
+ ## Common Requests
+
+ $rsp  = $dc->request($request);
+ $data = $dc->requestJson($request);
+
+ $version = $dc->version();
+ $status  = $dc->status();
+ $vstatus = $dc->vstatus();
+ $info    = $dc->info();
+
+ $rsp     = $dc->expand_terms(\@pipeline, \@terms);
+ @terms   = $dc->expand(\@pipeline, \@terms);
+
+ ##---------------------------------------------------------------------
+ ## Low-level Communications
 
  $dc->send(@command);               ##-- send a command (prepends size)
  $dc->sendfh($fh,@command);         ##-- ... to specified filehandle
@@ -600,6 +721,8 @@ DDC::Client - Client socket utilities for DDC::Concordance
  $hits = $dc->parseTableData($buf); ##-- parse a return buffer in 'table' mode
  $hits = $dc->parseHtmlData($buf);  ##-- parse a return buffer in 'html' mode
 
+ @terms = $dc->parseExpandTermsResponse($buf);  ##-- parse an expand_terms response buffer
+
 =cut
 
 ##======================================================================
@@ -655,16 +778,17 @@ e.g. by setting:
 =item accepted %args:
 
  (
-  optFile  =>$filename,      ##-- parse meta names, separators from DDC *.opt file
-  connect  =>\%connectArgs,  ##-- passed to IO::Socket::INET->new()
-  mode     =>$mode,          ##-- query mode; one of qw(json table text html raw); default='json'
-  parseMeta=>$bool,          ##-- if true, hit metadata will be parsed to $hit->{_meta} (default=1)
-  parseContext=>$bool,       ##-- if true, hit context data will be parsed to $hit->{_ctx} (default=1)
-  keepRaw  =>$bool,          ##-- if false, raw context buffer $hit->{_raw} will be deleted after parsing context data (default=false)
-  encoding =>$enc,           ##-- DDC server encoding (default='UTF-8')
-  fieldSeparator => $str,    ##-- intra-token field separator (default="\x{1f}": ASCII unit separator); 'text' and 'table' modes only
-  tokenSeparator => $str,    ##-- inter-token separator       (default="\x{1e}": ASCII record separator); 'text' and 'table' modes only
-  metaNames      => \@names, ##-- metadata names for 'text' and 'html' modes; default=none
+  optFile  =>$filename,       ##-- parse meta names, separators from DDC *.opt file
+  connect  =>\%connectArgs,   ##-- passed to IO::Socket::INET->new()
+  linger   =>\@linger,        ##-- SO_LINGER socket option (default=[1,0]: immediate termination)
+  mode     =>$mode,           ##-- query mode; one of qw(json table text html raw); default='json'
+  parseMeta=>$bool,           ##-- if true, hit metadata will be parsed to $hit->{_meta} (default=1)
+  parseContext=>$bool,        ##-- if true, hit context data will be parsed to $hit->{_ctx} (default=1)
+  keepRaw  =>$bool,           ##-- if false, raw context buffer $hit->{_raw} will be deleted after parsing context data (default=false)
+  encoding =>$enc,            ##-- DDC server encoding (default='UTF-8')
+  fieldSeparator => $str,     ##-- intra-token field separator (default="\x{1f}": ASCII unit separator); 'text' and 'table' modes only
+  tokenSeparator => $str,     ##-- inter-token separator       (default="\x{1e}": ASCII record separator); 'text' and 'table' modes only
+  metaNames      => \@names,  ##-- metadata names for 'text' and 'html' modes; default=none
 
   textHighlight => [$l0,$r0,$l1,$r1],  ##-- highlighting strings, text mode (default=[qw(&& && _& &_)])
   htmlHighlight => [$l0,$r0,$l1,$r1],  ##-- highlighting strings, html mode (default=[('<STRONG><FONT COLOR=red>','</FONT></STRONG>') x 2])
@@ -710,10 +834,87 @@ Closes the underlying socket if currently open.
 =cut
 
 ##----------------------------------------------------------------
-## DESCRIPTION: DDC::Client: Query: print(), read*()
+## DESCRIPTION: DDC::Client: Common Requests
 =pod
 
-=head2 Query: print(), read*()
+=head2 Common Requests
+
+=over 4
+
+=item request
+
+ $rsp = $dc->request($request_string);
+
+Send a raw DDC request and return the server's response as a raw byte-string.
+
+=item requestJson
+
+ $data = $dc->requestJson($request_string);
+
+Send a raw DDC request and decode the server's response as JSON data.
+
+=item version
+
+ $server_version = $dc->version();
+
+Request the current running version of the selected server,
+wraps $dc-E<gt>L<request|/request>("version").
+
+=item status
+
+ $status = $dc->status();
+ $status = $dc->status($timeout);
+
+Get basic server status;
+wraps $dc-E<gt>L<requestJson|/requestJson>("status $timeout").
+
+=item vstatus
+
+ $vstatus = $dc->vstatus();
+ $vstatus = $dc->vstatus($timeout);
+
+Get verbose server status;
+wraps $dc-E<gt>L<requestJson|/requestJson>("vstatus $timeout").
+
+=item info
+
+ $info = $dc->info();
+ $info = $dc->info($timeout);
+
+Get verbose server information;
+wraps $dc-E<gt>L<requestJson|/requestJson>("info $timeout").
+
+=item expand_terms
+
+ $expandRaw = $dc->expand_terms($pipeline, $term);
+ $expandRaw = $dc->expand_terms($pipeline, $term, $timeout);
+ $expandRaw = $dc->expand_terms($pipeline, $term, $timeout, $subcorpus);
+
+Perform server-side term-expansion for the term C<$term> via pipeline C<$pipeline>.
+Both C<$term> and C<$pipeline> may be specified as ARRAY-refs or bare strings.
+Returns the raw response data string.
+
+=item expand
+
+ @terms = $dc->expand($pipeline, $term);
+ @terms = $dc->expand($pipeline, $term, $timeout);
+ @terms = $dc->expand($pipeline, $term, $timeout, $subcorpus);
+
+Perform server-side term-expansion for the term C<$term> via pipeline C<$pipeline>
+and parses the response with L<parseExpandTermsResponse|/parseExpandTermsResponse>.
+Returns an array C<@terms> of server expansions in list-context;
+in scalar context returns the reference \@terms to such an array.
+
+=back
+
+=cut
+
+
+##----------------------------------------------------------------
+## DESCRIPTION: DDC::Client: Low-level Communications
+=pod
+
+=head2 Low-level Communications
 
 =over 4
 
@@ -790,6 +991,16 @@ the individual hits.
 
 JSON parsing requires the L<JSON|JSON> module.
 
+=item parseExpandTermsResponse
+
+ \@terms = $dc->parseExpandTermsResponse($buf);
+  @terms = $dc->parseExpandTermsResponse($buf);
+
+Parses a DDC server C<expand_terms> response buffer.
+Returns an array C<@terms> of server expansions in list-context;
+in scalar context returns the reference \@terms to such an array.
+
+
 =back
 
 =cut
