@@ -97,6 +97,8 @@ sub new {
   return $that->_new(Label=>$opName,OpName=>$opName,Dtr1=>$dtr1,Dtr2=>$dtr2,Negated=>$negated,%opts);
 }
 
+sub Children { [grep {defined($_)} @{$_[0]}{qw(Dtr1 Dtr2)}]; }
+
 sub GetMatchId {
   my $obj = shift;
   return (($obj->{Dtr2} && $obj->{Dtr2}->GetMatchId)
@@ -507,6 +509,8 @@ sub new {
   return $that->SUPER::new("NEAR",0,Dist=>(defined($dist) ? $dist : 1), Dtr1=>$dtr1, Dtr2=>$dtr2, Dtr3=>$dtr3, %opts);
 }
 
+sub Children { [grep {defined($_)} @{$_[0]}{qw(Dtr1 Dtr2 Dtr3)}]; }
+
 sub Clear { delete @{$_[0]}{qw(Dtr1 Dtr2 Dtr3)}; }
 
 sub GetMatchId {
