@@ -5,7 +5,7 @@ package DDC::PP;
 
 ##--------------------------------------------------------------
 ## ddcConfig.h
-sub library_version { return $VERSION; }
+sub library_version { return "2.1.1"; }
 
 
 ##--------------------------------------------------------------
@@ -29,7 +29,7 @@ our @HitSortEnumStrings = qw(
 			      asc_by_size desc_by_size
 			      asc desc
 			      asc_by_rank desc_by_rank
-			      asc_context desc_context
+			      asc_middle desc_middle
 			      asc_left desc_left
 			      asc_right desc_right
 			      random
@@ -81,7 +81,7 @@ DDC::PP::Constants - pure-perl DDC::XS clone: constants
  ##---------------------------------------------------------------------
  ## Top-Level constants (not exported)
  
- $string = DDC::PP::library_version()
+ $string = DDC::PP::library_version();
  
  ##---------------------------------------------------------------------
  ## ConcCommon.h: HitSortEnum (export-tag ":hitsort")
@@ -128,7 +128,7 @@ Bryan Jurish E<lt>moocow@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2016 by Bryan Jurish
+Copyright (C) 2016-2017 by Bryan Jurish
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.14.2 or,
