@@ -4,7 +4,7 @@
 ## Author: Bryan Jurish <moocow@cpan.org>
 ## Description:
 ##  + lexer for ddc queries (formerly DDC::Query::yylexer)
-##  + last updated for ddc v2.0.43
+##  + last updated for ddc v2.1.1
 ##======================================================================
 
 package DDC::PP::yyqlexer;
@@ -26,35 +26,30 @@ our @ISA = qw();
 ##   + common shared regex definitions
 our (%DEF);
 BEGIN {
-  %DEF =
-    (
-     ##-- copied from ddc-2.0.43/src/ConcordLib/yyQLexer.l ; extra escapes needed for backslashes ('\\' -> '\\\\')
-     'ws' => '[ \t\n\r\f\x0b]',
-     'punct' => '[\^\!\~\$\@\%\/\=\?\_\:\;\+\#\.\\\\\*\,\'\"\&\|\(\)\[\]\{\}\<\>]',
+  ##-- adapted from ddc-2.1.1/src/ConcordLib/yyQLexer.l ; extra escapes needed for backslashes ('\\' -> '\\\\')
 
-     int_boundary => '[ \t\n\r\f\x0b\0\^\&\|\!\(\)\[\]\{\}\<\>\'\"\,\;\:\#\=\*\%\~\\\\\$\@\/\?\_\+\-\.]',
-     kw_boundary  => '[ \t\n\r\f\x0b\0\^\&\|\!\(\)\[\]\{\}\<\>\'\"\,\;\:\#\=\*\%\~\\\\\$\@\/\?\_\+\-\.]',
+  ##-- whitespace
+  $DEF{ws} = '[ \t\n\r\f\x0b]';
 
-     ssymbol_cfirst => '[^ \t\r\n\f\x0b\0\^\&\|\!\(\)\[\]\{\}\<\>\'\"\,\;\:\#\=\*\%\\\\\~\/\$\@\?\_\+\.0-9]',
-     ssymbol_crest  => '[^ \t\r\n\f\x0b\0\^\&\|\!\(\)\[\]\{\}\<\>\'\"\,\;\:\#\=\*\%\\\\\~\/]',
-     symbol_cescape => '(?:\\\\.)',
+  ##-- integer boundary characters
+  $DEF{int_boundary} = '[ \t\n\r\f\x0b\0&|!?^%,:;#*=~(){}<>\[\]\/\\\\\'\".$@_+-]';
 
-     isymbol_cfirst  => '[0-9]',
-     isymbol_crest   => '[^ \t\r\n\f\x0b\0\^\&\|\!\(\)\[\]\{\}\<\>\'\"\,\;\:\#\=\*\%\\\\\~\/\$\@\?\_\+\-\.]',
+  ##-- bareword symbols
+  $DEF{symbol_cfirst}  = '[^ \t\n\r\f\x0b\0&|!?^%,:;#*=~(){}<>\[\]\\\\\/\'\".$@]';
+  $DEF{symbol_crest}   = '[^ \t\n\r\f\x0b\0&|!?^%,:;#*=~(){}<>\[\]\\\\\/\'\"]';
+  $DEF{symbol_cescape} = '(?:\\\\.)';
+  $DEF{symbol_text}    = "(?:$DEF{symbol_cescape}|$DEF{symbol_cfirst})(?:$DEF{symbol_cescape}|$DEF{symbol_crest})*";
 
-     'int_boundary' => '[ \t\n\r\f\x0b\0\^\!\~\$\@\%\/\=\?\_\:\;\+\#\.\\\\\*\,\'\"\&\|\(\)\[\]\{\}\<\>]',
-    );
+  ##-- bareword index names (underscore and digits ok, but no '.', '-', or '+')
+  $DEF{index_char} = '[^ \t\n\r\f\x0b\0&|!?^%,:;#*=~(){}<>\[\]\\\\/\'\".$@+-]';
+  $DEF{index_name} = "(?:$DEF{index_char}|$DEF{symbol_cescape})+";
 
-  $DEF{ssymbol_text} = "(?:$DEF{symbol_cescape}|$DEF{ssymbol_cfirst})(?:$DEF{symbol_cescape}|$DEF{ssymbol_crest})*";
-  $DEF{isymbol_text} = "(?:$DEF{isymbol_cfirst}(?:$DEF{symbol_cescape}|$DEF{isymbol_crest})*)";
-  $DEF{symbol_text}  = "(?:$DEF{ssymbol_text}|$DEF{isymbol_text})";
+  ##-- single-quoted symbols
+  $DEF{sq_text} = "(?:[^\']|$DEF{symbol_cescape})*";
 
-  $DEF{index_name}   = $DEF{symbol_text};
-
-  $DEF{sq_text}      = "(?:[^\\']|$DEF{symbol_cescape})*";
-
-  $DEF{regex_text}  = '(?:(?:\\\\.)|[^\/\\\\])*';
-  $DEF{regex_modifier} = '[gimsx]';
+  ##-- regexes
+  $DEF{regex_text}     = "(?:(?:\\\\.)|[^\\\\/])*";
+  $DEF{regex_modifier} = '[dgimsx]';
 
   ##-- compile patterns
   foreach (keys %DEF) {
@@ -289,7 +284,7 @@ sub yylex {
       @part = qw();
 
       ##------------------------
-      if ($lex->{state} eq 'INITIAL') {
+      if ($lex->{state} eq 'INITIAL' || $lex->{state} eq 'Q_MATCHID') {
 
 	##-- end-of-file (should be first pattern)
 	if    ($$bufr =~ m/\G\z/)	{ $type = '__EOF__'; }
@@ -314,6 +309,7 @@ sub yylex {
 	elsif ($$bufr =~ m/\G\#(?:with)?in/pi)				{ $type = 'WITHIN'; }
 	elsif ($$bufr =~ m/\G\#(?:sep(?:arate)?|nojoin)(?:_hits)?/pi)	{ $type = 'SEPARATE_HITS'; }
 	elsif ($$bufr =~ m/\G\#(?:nosep(?:arate)?|join)(?:_hits)?/pi)	{ $type = 'NOSEPARATE_HITS'; }
+	elsif ($$bufr =~ m/\G\#(?:is_|has_)?date/pi)			{ $type = 'IS_DATE'; }
 	elsif ($$bufr =~ m/\G\#has(?:_field)?/pi)			{ $type = 'HAS_FIELD'; }
 	elsif ($$bufr =~ m/\G\#file(?:_?)names/pi)			{ $type = 'FILENAMES_ONLY'; }
 	elsif ($$bufr =~ m/\G\#debug_rank/pi)				{ $type = 'DEBUG_RANK'; }
@@ -321,7 +317,6 @@ sub yylex {
 	elsif ($$bufr =~ m/\G\#(?:less|asc)(?:_by)?_rank/pi)		{ $type = 'LESS_BY_RANK'; }
 	elsif ($$bufr =~ m/\G\#(?:greater|de?sc)(?:_by)?_date/pi)	{ $type = 'GREATER_BY_DATE'; }
 	elsif ($$bufr =~ m/\G\#(?:less|asc)(?:_by)?_date/pi)		{ $type = 'LESS_BY_DATE'; }
-	elsif ($$bufr =~ m/\G\#(?:is_|has_)?date/pi)			{ $type = 'IS_DATE'; }
 	elsif ($$bufr =~ m/\G\#(?:greater|de?sc)(?:_by)?_size/pi)	{ $type = 'GREATER_BY_SIZE'; }
 	elsif ($$bufr =~ m/\G\#(?:less|asc)(?:_by)?_size/pi)		{ $type = 'LESS_BY_SIZE'; }
 	elsif ($$bufr =~ m/\G\#(?:is_|has_)?size/pi)			{ $type = 'IS_SIZE'; }
@@ -350,16 +345,16 @@ sub yylex {
 	elsif ($$bufr =~ m/\Gs\/($DEF{regex_text})\//po)				{ $type='REGEX_SEARCH'; $text=$1; $lex->{state}='Q_REGREP'; }
 
 	##-- punctutation & special characters
-	elsif ($$bufr =~ m/\G\#=/p)				{ $type = 'HASH_EQUAL'; }	##-- hash+equal: exact distance
-	elsif ($$bufr =~ m/\G\#</p)				{ $type = 'HASH_LESS'; }	##-- hash+less: max distance
-	elsif ($$bufr =~ m/\G\#>/p)				{ $type = 'HASH_GREATER'; }	##-- hash+greater: min distance
+	elsif ($$bufr =~ m/\G#=/p)				{ $type = 'HASH_EQUAL'; }	##-- hash+equal: exact distance
+	elsif ($$bufr =~ m/\G#</p)				{ $type = 'HASH_LESS'; }	##-- hash+less: max distance
+	elsif ($$bufr =~ m/\G#>/p)				{ $type = 'HASH_GREATER'; }	##-- hash+greater: min distance
 	elsif ($$bufr =~ m/\G\$\./p)				{ $type = 'DOLLAR_DOT'; }	##-- positional anchor pseudo-index
-	elsif ($$bufr =~ m/\G\:\{/p)				{ $type = 'COLON_LBRACE'; }	##-- theusaurus-query operator
-	elsif ($$bufr =~ m/\G\@\{/p)				{ $type = 'AT_LBRACE'; }	##-- literal-set operator
-	elsif ($$bufr =~ m/\G\*\{/p)				{ $type = 'STAR_LBRACE'; }	##-- prefix-set opener
-	elsif ($$bufr =~ m/\G\}\*/p)				{ $type = 'RBRACE_STAR'; }	##-- suffix-set closer
-	elsif ($$bufr =~ m/\G[\!\.\,\:\;\$\@\%\^\#\=\~\/]/p)	{ $type = ${^MATCH}; } ##-- single-char punctuation operators
-	elsif ($$bufr =~ m/\G[\[\]\{\}\(\)\<\>]/p)		{ $type = ${^MATCH}; } ##-- parentheses
+	elsif ($$bufr =~ m/\G\:{/p)				{ $type = 'COLON_LBRACE'; }	##-- theusaurus-query operator
+	elsif ($$bufr =~ m/\G\@{/p)				{ $type = 'AT_LBRACE'; }	##-- literal-set operator
+	elsif ($$bufr =~ m/\G\*{/p)				{ $type = 'STAR_LBRACE'; }	##-- prefix-set opener
+	elsif ($$bufr =~ m/\G}\*/p)				{ $type = 'RBRACE_STAR'; }	##-- suffix-set closer
+	elsif ($$bufr =~ m/\G[!.,:;=@%^#~\/]/p)			{ $type = ${^MATCH}; } ##-- single-char punctuation operators
+	elsif ($$bufr =~ m/\G[\[\]{}()<>]/p)			{ $type = ${^MATCH}; } ##-- parentheses
 	elsif ($$bufr =~ m/\G\"/p)				{ $type = ${^MATCH}; } ##-- double-quotes
 
 	##-- truncated symbols
@@ -371,14 +366,12 @@ sub yylex {
 	elsif ($$bufr =~ m/\G($DEF{symbol_text})\*/po)	 { $type='PREFIX'; $text=$1; }	##-- right-truncated bareword (prefix symbol)
 	elsif ($$bufr =~ m/\G\*($DEF{symbol_text})/po)	 { $type='SUFFIX'; $text=$1; }	##-- left-truncated bareword (suffix symbol)
 
-	##-- numeric tokens
-	elsif ($$bufr =~ m/\G\'([\+\-]?[0-9]+)\'/p)			{ $type = 'INTEGER'; $text=$1; }
-	elsif ($$bufr =~ m/\G[\+\-]?[0-9]+(?=$DEF{int_boundary})/po)	{ $type = 'INTEGER'; }
-	elsif ($$bufr =~ m/\G[\+\-]?[0-9]+\z/p)				{ $type = 'INTEGER'; }
+	##-- index names (special handling to allow count(foo) #by[$w-1]
+	elsif ($$bufr =~ m/\G\$($DEF{index_name})/po)	{ $type='INDEX'; $text=$1; }
+	elsif ($$bufr =~ m/\G\$'($DEF{index_name})'/po)	{ $type='INDEX'; $text=$1; }
+	elsif ($$bufr =~ /m\G\$/p)			{ $type='$'; }
 
-	elsif ($$bufr =~ m/\G\'([0-9\-]+)\'/p)					{ $type = 'DATE'; $text=$1 }
-	elsif ($$bufr =~ m/\G[0-9]{4,}[0-9\-]+(?=$DEF{int_boundary})/po) 	{ $type = 'DATE'; }
-	elsif ($$bufr =~ m/\G[0-9]{4,}[0-9\-]+\z/po) 				{ $type = 'DATE'; }
+	##-- numeric tokens (handled below with symbols for perl lexer w/o longest-match)
 
 	##-- single-term wildcard
 	elsif ($$bufr =~ m/\G\*/p)     			{ $type = '*'; }
@@ -386,16 +379,34 @@ sub yylex {
 	##-- term expander pipelines
 	elsif ($$bufr =~ m/\G\|/p) 			{ $lex->{state}='Q_XPIPE'; $type='__SKIP__'; }
 
-	##-- symbols
-	elsif ($$bufr =~ m/\G\'($DEF{sq_text})\'/po)	{ $type = 'SYMBOL'; $text=$1; }
-	elsif ($$bufr =~ m/\G$DEF{symbol_text}/po)	{ $type = 'SYMBOL'; }
+	##-- symbols: quoted
+	elsif ($$bufr =~ m/\G\'($DEF{sq_text})\'/po) {
+	  $text = $1;
+	  if    ($text =~ m/^[\+\-]?[0-9]+\z/)			{ $type='INTEGER'; }
+	  elsif ($text =~ m/^[0-9-]+\z/)			{ $type='DATE'; }
+	  else							{ $type='SYMBOL'; }
+	}
+	##-- symbols: barewords
+	elsif ($lex->{state} eq 'INITIAL' && $$bufr =~ m/\G$DEF{symbol_text}/po) {
+	  $text  = ${^MATCH};
+	  $match = ${^MATCH};
+	  if    ($text =~ m/^[\+\-]?[0-9]+\z/)			{ $type='INTEGER'; }
+	  elsif ($text =~ m/^[0-9-]+\z/)			{ $type='DATE'; }
+	  else							{ $type='SYMBOL'; }
+	}
+	##-- barewords: integers and dates (Q_MATCHID)
+	elsif ($lex->{state} ne 'INITIAL' && $$bufr =~ m/\G[\+\-]?[0-9]+(?=$DEF{int_boundary})/po)	{ $type='INTEGER'; }
+	elsif ($lex->{state} ne 'INITIAL' && $$bufr =~ m/\G[\+\-]?[0-9]+\z/p)				{ $type='INTEGER'; }
+
+	elsif ($lex->{state} ne 'INITIAL' && $$bufr =~ m/\G[0-9]{4,}[0-9\-]+(?=$DEF{int_boundary})/po)	{ $type='DATE'; }
+	elsif ($lex->{state} ne 'INITIAL' && $$bufr =~ m/\G[0-9]{4,}[0-9\-]+\z/po)			{ $type='DATE'; }
 
 	##-- misc
 	elsif ($$bufr =~ m/\G\s+/p)	{ $type = '__SKIP__'; }
 	#elsif ($$bufr =~ m/\G./p)	{ $type = 'SYMBOL'; }
 	elsif ($$bufr =~ m/\G./p)	{ $type = '__ERROR__'; }
 
-	$match = ${^MATCH};
+	$match = ${^MATCH} if (!defined($match));
       }
       ##------------------------
       elsif ($lex->{state} eq 'Q_DATE') {
@@ -514,7 +525,7 @@ Bryan Jurish E<lt>moocow@cpan.orgE<gt>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011-2016 by Bryan Jurish
+Copyright (C) 2011-2017 by Bryan Jurish
 
 This package is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.14.2 or,
